@@ -620,16 +620,13 @@ disc_drive_pool_add_drive (ThDiscDrivePool *disc_pool, const gchar *udi)
 
 	drive = th_disc_drive_new (udi, device, vendor, product);
 
-	g_datalist_set_data_full (&disc_pool->priv->drives, udi, drive, g_object_unref);
-	
-	g_signal_emit (disc_pool, ddp_signals[DRIVE_ADDED], 0, drive);
-
-	if (req_polling)
-	{
-		g_object_set (drive, 
-		              "poll-media-required", TRUE, 
-		              "poll-media", TRUE, 
-		              NULL);
+	if (th_disc_drive_pool_add_drive (disc_pool, drive)) {	
+	  if (req_polling) {
+	    g_object_set (drive, 
+	        "poll-media-required", TRUE, 
+	        "poll-media", TRUE, 
+	        NULL);
+	  }
 	}
 
 	if (disc_pool->priv->find_timeout > 0)
@@ -640,6 +637,8 @@ disc_drive_pool_add_drive (ThDiscDrivePool *disc_pool, const gchar *udi)
 	g_free (device);
 	g_free (vendor);
 	g_free (product);
+
+	g_object_unref (drive); /* disc_pool->priv->drives kept a ref */
 }
 
 
@@ -1070,3 +1069,47 @@ disc_drive_pool_no_drives_cb (ThDiscDrivePool *ddpool)
 
 	return FALSE; /* only run once */
 }
+
+/***************************************************************************
+ *
+ *   th_disc_drive_pool_get_drive_from_udi
+ *
+ ***************************************************************************/
+
+ThDiscDrive *
+th_disc_drive_pool_get_drive_from_udi (ThDiscDrivePool * pool, const gchar * udi)
+{
+  g_return_val_if_fail (TH_IS_DISC_DRIVE_POOL (pool), NULL);
+  g_return_val_if_fail (udi != NULL, NULL);
+
+  return disc_drive_pool_get_drive_from_udi (pool, udi);
+}
+
+/***************************************************************************
+ *
+ *   th_disc_drive_pool_add_drive
+ *
+ ***************************************************************************/
+
+gboolean
+th_disc_drive_pool_add_drive (ThDiscDrivePool * pool, ThDiscDrive * drive)
+{
+  gchar *drive_udi = NULL;
+
+  g_return_val_if_fail (TH_IS_DISC_DRIVE_POOL (pool), FALSE);
+  g_return_val_if_fail (TH_IS_DISC_DRIVE (drive), FALSE);
+
+  g_object_get (drive, "udi", &drive_udi, NULL);
+  g_return_val_if_fail (drive_udi != NULL, FALSE);
+
+  if (disc_drive_pool_get_drive_from_udi (pool, drive_udi))
+    return FALSE;
+
+  g_datalist_set_data_full (&pool->priv->drives, drive_udi,
+      g_object_ref (drive), g_object_unref);
+	
+  g_signal_emit (pool, ddp_signals[DRIVE_ADDED], 0, drive);
+
+  return TRUE;
+}
+
