@@ -626,7 +626,7 @@ job_progress_calc_time_left (ThJobProgressDialog *jpd, guint pos_msecs)
 
 	/* note: secs_left might be negative, as length is not exact */
 	cstr = job_get_human_time_interval_str ((gint) secs_left);
-	eta = time (NULL) + (time_t) MAX (15, secs_left);
+	eta = time (NULL) + (time_t) MAX (15.0, secs_left);
 	strftime (timestr, sizeof (timestr), "%R", localtime (&eta));
 	th_label_set_text (jpd->priv->timeleft_label, "%s [%sh]", cstr, timestr); 
 }
@@ -1045,6 +1045,7 @@ job_progress_dialog_set_job_progress_bar (ThJobProgressDialog *jpd, guint n, gui
 gboolean
 th_job_progress_dialog_run_jobs (ThJobProgressDialog *jpd, GList *jobs)
 {
+	GObject  *power_manager;
 	gboolean  got_cancelled = FALSE;
 	guint     num_jobs, n = 0;
 	GList    *l;
@@ -1052,9 +1053,8 @@ th_job_progress_dialog_run_jobs (ThJobProgressDialog *jpd, GList *jobs)
 	g_return_val_if_fail (TH_IS_JOB_PROGRESS_DIALOG (jpd), FALSE);
 	g_return_val_if_fail (jobs != NULL, FALSE);
 
-	g_object_set (jpd->priv->power_manager_agent,
-	              "inhibit-power-manager", TRUE,
-	              NULL);
+ 	power_manager = g_object_ref (jpd->priv->power_manager_agent);
+	g_object_set (power_manager, "inhibit-power-manager", TRUE, NULL);
 
 	gtk_widget_hide (jpd->priv->close_button);
 	gtk_widget_show (jpd->priv->pause_button);
@@ -1081,7 +1081,8 @@ th_job_progress_dialog_run_jobs (ThJobProgressDialog *jpd, GList *jobs)
 		if (!job_progress_dialog_run_job (jpd, job, &got_cancelled)  &&  got_cancelled)
 		{
 			/* the dialog object has been destroyed now */
-			return FALSE;
+			jpd = NULL;
+			goto done;
 		}
 		
 		++n;
@@ -1104,9 +1105,11 @@ th_job_progress_dialog_run_jobs (ThJobProgressDialog *jpd, GList *jobs)
 	
 	gtk_widget_hide (GTK_WIDGET (jpd));
 
-	g_object_set (jpd->priv->power_manager_agent,
-	              "inhibit-power-manager", FALSE,
-	              NULL);
+done:
 
-	return TRUE;
+	g_object_set (power_manager, "inhibit-power-manager", FALSE, NULL);
+	g_object_unref (power_manager);
+
+	return (jpd != NULL);
 }
+
