@@ -56,7 +56,8 @@ enum
 	PROP_VIDEO_BITRATE,         /* in kbps */
 	PROP_AUDIO_BITRATE,         /* in kbps */
 	PROP_AUDIO_STREAM_ID,
-	PROP_NUM_AUDIO_STREAMS,
+	PROP_AUDIO_ALL,             /* Flag whether to record all streams */
+	PROP_NUM_SUB_STREAMS,
 	PROP_TARGET_OUTPUT_SIZE,    /* in MB   */
 	PROP_TARGET_QUALITY,
 	PROP_BYTES_WRITTEN,         /* bytes written, accessed from multiple threads          */
@@ -93,12 +94,15 @@ struct _ThJobPrivate
 
 	gchar         *title_id; /* unique ID string identifying disc and title */
 
-	GList         *audio_streams; /* list of ThJobAudioStream structs */
+	GList         *sub_streams;   /* list of ThJobSubStream structs   */
 
 	guint          video_bitrate;
 	guint          audio_bitrate;
 	
-	guint          audio_stream_id; /* logical audio stream number */
+	guint          audio_stream_id; /* logical audio stream selected */
+	guint          sub_stream_id;   /* logical subpicture stream selected
+	                                 * (G_MAXUINT = NONE) */
+	gboolean       audio_all; /* Set if all audiostreams should be recorded */
 
 	guint          crop_left, crop_right;
 	guint          crop_top, crop_bottom;
@@ -111,8 +115,8 @@ struct _ThJobPrivate
 
 	GstElement    *pipeline;
 	
-	gchar         *audio_pad_name;
-	gchar         *video_pad_name;
+	const gchar   *audio_pad_name;   /* interned string */
+	const gchar   *video_pad_name;   /* interned string */
 	const gchar   *audio_decoder;
 
 	guint          bytes_total;
@@ -137,14 +141,12 @@ struct _ThJobPrivate
 	gchar         *comment_tag;
 };
 
-typedef struct _ThJobAudioStream ThJobAudioStream;
-struct _ThJobAudioStream
+typedef struct
 {
-	guint  aid;
+	guint  sid;
 	gchar *lang;
 	gchar *pad_name;
-	gchar *desc;
-};
+} ThJobSubStream;
 
 static void             job_class_init       (ThJobClass *klass);
 
@@ -156,7 +158,9 @@ static GdkPixbuf       *job_get_snapshot_from_current_buf (ThJob *j);
 
 static void             job_set_video_bitrate_from_file_size (ThJob *j, gdouble targetsize_megabyte);
 
-static gdouble          job_get_output_size_estimate (ThJob *j, guint audio_bitrate, guint video_bitrate, guint length);
+static gdouble          job_get_output_size_estimate (ThJob *j, guint video_bitrate);
+
+static guint            job_get_total_audio_bitrate (ThJob *j);
 
 static gdouble          job_calculate_weight (ThJob *j);
 
@@ -167,20 +171,6 @@ static GObjectClass    *j_parent_class;           /* NULL */
 
 static guint            j_signals[LAST_SIGNAL];   /* all 0 */
 
-/***************************************************************************
- *
- *   job_aid_cmp_func
- *
- ***************************************************************************/
-
-static gint
-job_aid_cmp_func (const ThJobAudioStream *audio_stream, gconstpointer aid)
-{
-	g_return_val_if_fail (audio_stream != NULL, -1);
-
-	return (gint) (audio_stream->aid - GPOINTER_TO_UINT (aid));
-}
-
 /***************************************************************************
  *
  *   job_get_audio_stream_from_aid
@@ -190,16 +180,17 @@ job_aid_cmp_func (const ThJobAudioStream *audio_stream, gconstpointer aid)
 static ThJobAudioStream *
 job_get_audio_stream_from_aid (ThJob *j, guint aid)
 {
-	GList *l;
-	
-	l = g_list_find_custom (j->priv->audio_streams, 
-	                        GUINT_TO_POINTER (aid), 
-	                        (GCompareFunc) job_aid_cmp_func);
-	
-	if (l == NULL)
-		return NULL;
+	guint i;
 
-	return (ThJobAudioStream *) l->data;
+	for (i = 0; i < j->audio_streams->len; ++i) {
+		ThJobAudioStream *as;
+
+		as = &g_array_index (j->audio_streams, ThJobAudioStream, i);
+		if (as->aid == aid)
+			return as;
+	}
+
+	return NULL;
 }
 
 /***************************************************************************
@@ -214,7 +205,7 @@ job_set_property (GObject      *object,
                   const GValue *value,
                   GParamSpec   *pspec)
 {
-	gboolean  need_size_est = FALSE;
+	gboolean  need_size_est = FALSE; /* FIXME: this is unused */
 	ThJob    *j = (ThJob*) object;
 
 	g_return_if_fail (TH_IS_JOB (j));
@@ -271,7 +262,8 @@ job_set_property (GObject      *object,
 		{
 			ThJobAudioStream *as;
 			guint             new_aid;
-			
+			/* If you set a stream ID, you don't want all audio */
+			j->priv->audio_all = FALSE;
 			new_aid = g_value_get_uint (value);
 
 			as = job_get_audio_stream_from_aid (j, new_aid);
@@ -289,6 +281,11 @@ job_set_property (GObject      *object,
 			}
 			break;
 		}
+		case PROP_AUDIO_ALL:
+		{
+			j->priv->audio_all = g_value_get_boolean(value);
+			break;
+		}
 		case PROP_NUM_FRAMES_ENCODED:
 		{
 			th_atomic_uint_set (&j->priv->num_frames,
@@ -431,8 +428,12 @@ job_get_property (GObject      *object,
 			g_value_set_uint (value, j->priv->audio_stream_id);
 			break;
 
-		case PROP_NUM_AUDIO_STREAMS:
-			g_value_set_uint (value, g_list_length (j->priv->audio_streams));
+		case PROP_AUDIO_ALL:
+			g_value_set_boolean (value, j->priv->audio_all);
+			break;
+
+		case PROP_NUM_SUB_STREAMS:
+			g_value_set_uint (value, g_list_length (j->priv->sub_streams));
 			break;
 
 		case PROP_BYTES_WRITTEN:
@@ -620,11 +621,20 @@ job_class_init (ThJobClass *klass)
 	                                                    0, G_MAXUINT, G_MAXUINT, 
 	                                                    G_PARAM_READWRITE));
 
-	g_object_class_install_property (object_class, PROP_NUM_AUDIO_STREAMS,
-	                                 g_param_spec_uint ("num-audio-streams", 
-	                                                    "Number of Audio Streams", 
-	                                                    "Number of Audio Streams", 
-	                                                    0, G_MAXUINT, 0, 
+	g_object_class_install_property (object_class, PROP_AUDIO_ALL,
+	                                 g_param_spec_boolean ("audio-all", 
+	                                                    "Whether to encode all streams", 
+	                                                    "Whether to encode all streams", 
+	                                                    FALSE, 
+	                                                    G_PARAM_READWRITE));
+
+	/* FIXME: get rid of this useless property and replace with direct
+	 * structure access instead */
+	g_object_class_install_property (object_class, PROP_NUM_SUB_STREAMS,
+	                                 g_param_spec_uint ("num-sub-streams", 
+	                                                    "Number of Subpicture Streams", 
+	                                                    "Number of Subpicture Streams", 
+	                                                    0, 32, 0, 
 	                                                    G_PARAM_READABLE));
 
 	g_object_class_install_property (object_class, PROP_TARGET_OUTPUT_SIZE,
@@ -765,6 +775,9 @@ job_instance_init (ThJob *j)
 	j->priv = g_new0 (ThJobPrivate, 1);
 	
 	j->priv->audio_stream_id = G_MAXUINT;
+	j->priv->sub_stream_id = G_MAXUINT;
+
+	j->audio_streams = g_array_new (TRUE, TRUE, sizeof (ThJobAudioStream));
 }
 
 /***************************************************************************
@@ -788,13 +801,12 @@ job_finalize (GObject *object)
 		gst_object_unref (GST_OBJECT (j->priv->pipeline));
 	}
 
-	g_free (j->priv->audio_pad_name);
-	g_free (j->priv->video_pad_name);
-	
 	memset (j->priv, 0xab, sizeof (ThJobPrivate));
 	g_free (j->priv);
 	j->priv = NULL;
 
+	g_array_free (j->audio_streams, TRUE);
+
 	/* chain up */
 	j_parent_class->finalize (object);
 }
@@ -1227,16 +1239,17 @@ th_job_get_effective_picture_size (ThJob         *job,
  ***************************************************************************/
 
 static gdouble
-job_get_output_size_estimate (ThJob *j, guint audio_bitrate, guint video_bitrate, guint length)
+job_get_output_size_estimate (ThJob *j, guint video_bitrate)
 {
 	gdouble  total_bitrate_kbps, total_size_kbit, est_output_size;
 	
-	total_bitrate_kbps = (gdouble) (video_bitrate + audio_bitrate);
+	total_bitrate_kbps = (gdouble) video_bitrate;
+	total_bitrate_kbps +=  (gdouble) job_get_total_audio_bitrate (j);
 	
 	/* Add 1.75% overhead for the Ogg encapsulation (ca. 1-2% apparently) */
 	total_bitrate_kbps = total_bitrate_kbps * 1.0175;
 	
-	total_size_kbit = total_bitrate_kbps * (gdouble) length;
+	total_size_kbit = total_bitrate_kbps * (gdouble) j->priv->title_length;
 
 	est_output_size = total_size_kbit / (8.0 * 1024.0);
 
@@ -1247,6 +1260,30 @@ job_get_output_size_estimate (ThJob *j, guint audio_bitrate, guint video_bitrate
 	return est_output_size;
 }
 
+/***************************************************************************
+ *
+ *   job_get_total_audio_bitrate
+ *
+ *   Returns the sum of the bitrates of all (selected) audio streams
+ *
+ ***************************************************************************/
+
+static guint
+job_get_total_audio_bitrate (ThJob * job)
+{
+  guint num_audio_streams;
+
+  /* when we support audio bitrate configuration and audio encoder selection
+   * etc. we need to do something more sophisticated here */
+  if (job->priv->audio_all) {
+    num_audio_streams = job->audio_streams->len;
+  } else {
+    num_audio_streams = 1;
+  }
+
+  return num_audio_streams * job->priv->audio_bitrate;
+}
+
 /***************************************************************************
  *
  *   job_set_video_bitrate_from_file_size
@@ -1260,7 +1297,7 @@ job_set_video_bitrate_from_file_size (ThJob *j, gdouble targetsize_megabyte)
 
 	g_return_if_fail (TH_IS_JOB (j));
 	g_return_if_fail (targetsize_megabyte > 0.0);
-	
+
 	kilobits = targetsize_megabyte * 1024.0 * 8.0;
 	
 	kbps_total = kilobits / (gdouble) j->priv->title_length;
@@ -1269,7 +1306,7 @@ job_set_video_bitrate_from_file_size (ThJob *j, gdouble targetsize_megabyte)
 	 * (ca. 1-2% apparently; 1.2% in my test for a full movie) */
 	kbps_total = kbps_total * 0.9825;
 	
-	kbps_video = (guint) (kbps_total - (gdouble) j->priv->audio_bitrate);
+	kbps_video = kbps_total - (gdouble) job_get_total_audio_bitrate (j);
 
 	j->priv->target_output_size = targetsize_megabyte;
 	
@@ -1448,19 +1485,37 @@ job_video_probe_cb (GstPad *sink_pad, GstBuffer *buf, ThJob *j)
  ***************************************************************************/
 
 static void
-job_create_pipeline_setup_queues (GstBin *pipeline)
+job_create_pipeline_setup_queues (ThJob * j)
 {
-	/* unset/disable all limits apart from the bytes limit */
-	th_bin_set_child_properties (pipeline, "q-a-in",
-	              "max-size-buffers", (guint) 0,
-	              "max-size-bytes", (guint) 10 * 1024 * 1024,
-	              "max-size-time", (guint64) 0,
-	              NULL);
-	th_bin_set_child_properties (pipeline, "q-v-in",
-	              "max-size-buffers", (guint) 0,
-	              "max-size-bytes", (guint) 10 * 1024 * 1024,
-	              "max-size-time", (guint64) 0,
-	              NULL);
+  guint i;
+
+  /* configure audio in queues */
+  for (i = 0; i < j->audio_streams->len; ++i) {
+    ThJobAudioStream *as;
+
+    as = &g_array_index (j->audio_streams, ThJobAudioStream, i);
+
+    /* FIXME: use as->selected once the config dialog gets fixed up */
+    if (j->priv->audio_all || j->priv->audio_stream_id == as->aid) {
+      gchar enc_name[32];
+
+      g_snprintf (enc_name, sizeof (enc_name), "q-a-in-%u", as->aid);
+
+      /* unset/disable all limits apart from the bytes limit */
+      th_bin_set_child_properties (GST_BIN (j->priv->pipeline), enc_name,
+          "max-size-buffers", (guint) 0,
+          "max-size-bytes", (guint) 10 * 1024 * 1024,
+         "max-size-time", (guint64) 0,
+          NULL);
+    }
+  }
+
+  /* configure video in queue */
+  th_bin_set_child_properties (GST_BIN (j->priv->pipeline), "q-v-in",
+      "max-size-buffers", (guint) 0,
+      "max-size-bytes", (guint) 10 * 1024 * 1024,
+      "max-size-time", (guint64) 0,
+      NULL);
 }
 
 
@@ -1485,9 +1540,24 @@ job_probe_pads_new_pad_cb (ThJob *j, GstPad *new_pad, GstElement *dvddemux)
 
 	as = job_get_audio_stream_from_aid (j, j->priv->audio_stream_id);
 
+	/* FIXME: sanity check, remove again */
+	{
+	  ThJobAudioStream *as2;
+	  guint aid = G_MAXUINT;
+
+	  if (sscanf (pad_name, "audio_%u", &aid) == 1) {
+	    as2 = job_get_audio_stream_from_aid (j, aid);
+	    if (as2) {
+	      th_log ("Mapped caps %s => decoder %s",
+	              gst_caps_to_string (gst_pad_get_caps (new_pad)),
+	              as2->decoder);
+	    }
+	  }
+	}
+
 	if (j->priv->video_pad_name == NULL  &&  g_str_has_prefix (pad_name, "video_"))
 	{
-		j->priv->video_pad_name = g_strdup (pad_name);
+		j->priv->video_pad_name = g_intern_string (pad_name);
 		th_log ("dvddemux::new-pad '%s' (using)", pad_name);
 		goto hook_up;
 	}
@@ -1498,7 +1568,7 @@ job_probe_pads_new_pad_cb (ThJob *j, GstPad *new_pad, GstElement *dvddemux)
 		const gchar *mime = NULL;
 		GstCaps *caps;
 		
-		j->priv->audio_pad_name = g_strdup (pad_name);
+		j->priv->audio_pad_name = g_intern_string (pad_name);
 		j->priv->audio_decoder = "decodebin";
 		
 		caps = gst_pad_get_caps (new_pad);
@@ -1546,7 +1616,7 @@ hook_up:
 	fakesink = gst_bin_get_by_name (GST_BIN (pipeline), "dummyfakesink");
 	if (fakesink)
 	{
-		th_log ("REMOVING DUMMY FAKSINK"); /* FIXME */
+		th_log ("REMOVING DUMMY FAKESINK"); /* FIXME */
 		gst_element_set_state (fakesink, GST_STATE_NULL);
 		gst_bin_remove (GST_BIN (pipeline), fakesink);
 		gst_object_unref (fakesink);
@@ -1620,8 +1690,6 @@ job_probe_pads (ThJob *j, GError **err)
 			             j->priv->audio_stream_id);
 		}
 
-		g_free (j->priv->audio_pad_name);
-		g_free (j->priv->video_pad_name);
 		j->priv->audio_pad_name = NULL;
 		j->priv->video_pad_name = NULL;
 
@@ -1630,44 +1698,6 @@ job_probe_pads (ThJob *j, GError **err)
 
 	th_log ("Found audio and video pads.");
 
-	/* temporary hack: warn about a52dec <= 0.10.4.1 */
-	if (g_str_equal (j->priv->audio_decoder, "a52dec") &&
-	    !gst_default_registry_check_feature_version ("a52dec", 0, 10, 5)) {
-		gboolean have_0104_cvs = FALSE;
-
-	    if (gst_default_registry_check_feature_version ("a52dec", 0, 10, 4)) {
-			GstPlugin *plugin;
-
-			if ((plugin = gst_default_registry_find_plugin ("a52dec"))) {
-				const gchar *version = gst_plugin_get_version (plugin);
-
-				have_0104_cvs = g_str_has_prefix (version, "0.10.4.");
-				gst_object_unref (plugin);
-			}
-		}
-		if (!have_0104_cvs) {
-			GtkWidget *dlg;
-
-			dlg = gtk_message_dialog_new (NULL,
-			                              GTK_DIALOG_MODAL,
-			                              GTK_MESSAGE_INFO,
-			                              GTK_BUTTONS_OK,
-			                              _("Note"));
-
-			/* this is only a temporary notice until -ugly 0.10.5 is out */
-			gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg),
-			         _("Your version of the GStreamer AC3 decoder element "
-			           "(a52dec) might lead to too quiet audio output "
-			           "for some films. You should check that the audio "
-			           "output is acceptable on the resulting file. The "
-			           "problem has been fixed in newer versions of "
-			           "GStreamer gst-plugins-ugly."));
-
-			gtk_window_present (GTK_WINDOW (dlg));		
-			g_signal_connect (dlg, "response", G_CALLBACK (gtk_widget_destroy), NULL);
-		}
-	}
-
 	return TRUE;
 }
 
@@ -1706,10 +1736,9 @@ job_check_free_disk_space (ThJob *j)
 	return TRUE; /* call us again */
 }
 
-/* FIXME: why do we need to do this in the state change callback? */
 static void
-//job_pipeline_tag_setter_state_change_cb (ThJob *j, guint old_state, guint new_state, GstElement *element)
-job_pipeline_tag_setter_setup (ThJob *j, GstElement *element)
+job_pipeline_set_tags_on_encoder (ThJob *j, GstElement *element,
+    const gchar *lang)
 {
 	if (!GST_IS_TAG_SETTER (element))
 		return;
@@ -1734,6 +1763,11 @@ job_pipeline_tag_setter_setup (ThJob *j, GstElement *element)
 	                         GST_TAG_MERGE_APPEND, 
 	                         GST_TAG_COMMENT, "Created with thoggen: http://thoggen.net", 
 	                         NULL);
+
+	if (lang != NULL && *lang != '\0' && g_utf8_validate (lang, -1, NULL)) {
+	  gst_tag_setter_add_tags (GST_TAG_SETTER (element),
+	      GST_TAG_MERGE_APPEND, GST_TAG_LANGUAGE_CODE, lang, NULL);
+	}
 }
 
 /***************************************************************************
@@ -1746,34 +1780,30 @@ static void
 job_create_pipeline_setup_tagging (ThJob *j, GstBin *pipeline)
 {
 	GstElement *tagsetter;
-#if 0
-	/* FIXME: don't leak references returned by _bin_get_by_name */
-	g_signal_connect_swapped (gst_bin_get_by_name (pipeline, "mux"),
-	                          "state-change",
-	                          G_CALLBACK (job_pipeline_tag_setter_state_change_cb),
-	                          j);
+	guint i;
 
-	g_signal_connect_swapped (gst_bin_get_by_name (pipeline, "a-encoder"),
-	                          "state-change",
-	                          G_CALLBACK (job_pipeline_tag_setter_state_change_cb),
-	                          j);
-
-	g_signal_connect_swapped (gst_bin_get_by_name (pipeline, "v-encoder"),
-	                          "state-change",
-	                          G_CALLBACK (job_pipeline_tag_setter_state_change_cb),
-	                          j);
-#endif
 	/* FIXME: maybe do this by recursing into the pipeline
 	 * instead of picking out the elements by name? */
 	tagsetter = gst_bin_get_by_name (pipeline, "mux");
-	job_pipeline_tag_setter_setup (j, tagsetter);
-        gst_object_unref (tagsetter);
-	tagsetter = gst_bin_get_by_name (pipeline, "a-encoder");
-	job_pipeline_tag_setter_setup (j, tagsetter);
-        gst_object_unref (tagsetter);
+	job_pipeline_set_tags_on_encoder (j, tagsetter, NULL);
+	gst_object_unref (tagsetter);
+
+	for (i = 0; i < j->audio_streams->len; ++i) {
+	  ThJobAudioStream *as;
+	  gchar enc_name[32];
+
+	  as = &g_array_index (j->audio_streams, ThJobAudioStream, i);
+	  g_snprintf (enc_name, sizeof (enc_name), "a-encoder-%u", as->aid);
+	  tagsetter = gst_bin_get_by_name (pipeline, enc_name);
+	  if (tagsetter) {
+	    job_pipeline_set_tags_on_encoder (j, tagsetter, as->lang);
+            gst_object_unref (tagsetter);
+	  }
+	}
+
 	tagsetter = gst_bin_get_by_name (pipeline, "v-encoder");
-	job_pipeline_tag_setter_setup (j, tagsetter);
-        gst_object_unref (tagsetter);
+	job_pipeline_set_tags_on_encoder (j, tagsetter, NULL);
+	gst_object_unref (tagsetter);
 }
 
 /***************************************************************************
@@ -1832,6 +1862,29 @@ job_create_pipeline_setup_video_encoder (ThJob *j)
 	gst_object_unref (video_enc);
 }
 
+static void
+job_create_pipeline_setup_audio_encoders (ThJob * j)
+{
+  guint i;
+
+  for (i = 0; i < j->audio_streams->len; ++i) {
+    ThJobAudioStream *as;
+
+    as = &g_array_index (j->audio_streams, ThJobAudioStream, i);
+
+    /* FIXME: fix up configuration dialog so we can use as->selected in future */
+    if (j->priv->audio_all || j->priv->audio_stream_id == as->aid) {
+      gchar enc_name[32];
+
+      g_snprintf (enc_name, sizeof (enc_name), "a-encoder-%u", as->aid);
+
+      /* configure vorbisenc */
+      th_bin_set_child_properties (GST_BIN (j->priv->pipeline), enc_name,
+          "bitrate", j->priv->audio_bitrate * 1000, "managed", TRUE,  NULL);
+    }
+  }
+}
+
 /***************************************************************************
  *
  *   job_create_pipeline
@@ -1841,11 +1894,27 @@ job_create_pipeline_setup_video_encoder (ThJob *j)
 static gboolean
 job_create_pipeline (ThJob *j, GError **err)
 {
+	const gchar *deinterlacer, *scale_method;
 	GstElement  *pipe, *out_identity;
 	GstBus      *bus;
-	gchar       *pipe_desc;
+	GString     *pipe_desc;
 	guint        width, height;
 
+	if (gst_default_registry_check_feature_version ("ffdeinterlace", 0, 10, 2))
+	  deinterlacer = "ffdeinterlace";
+	else
+	  deinterlacer = "thdeinterlace";
+
+	th_log ("Using %s for deinterlacing", deinterlacer);
+
+	/* FIXME: always use 4-tap once we depend on -base 0.10.17 */
+	if (gst_default_registry_check_feature_version ("videoscale", 0, 10, 17))
+	  scale_method = "4-tap";
+	else
+	  scale_method = "bilinear";
+
+	th_log ("Scaling method: %s", scale_method);
+
 	if (!th_job_get_effective_picture_size (j, j->priv->picture_size, &width, &height))
 		g_return_val_if_reached (FALSE);
 	
@@ -1856,57 +1925,87 @@ job_create_pipeline (ThJob *j, GError **err)
 	        j->priv->picture_size, width, height, j->priv->crop_left,
 	        j->priv->crop_right, j->priv->crop_top, j->priv->crop_bottom);
 
-	pipe_desc = g_strdup_printf (
-	      "    oggmux name=mux                                                    "
-	      "  ! identity name=outidentity                                          "
-	      "  ! filesink name=filesink                                             " 
-	      "                                                                       "
-	      "      dvdreadsrc name=src                                              "
-	      "    ! dvddemux name=demux                                              "
-	      "                                                                       " 
-	      "          demux.%s                                                     "
-	      "        ! queue name=q-a-in                                            "
-	      "        ! %s                                                           "
-	      "        ! audioconvert                                                 "
-	      "        ! audioresample                                                "
-	      "        ! audio/x-raw-int,rate=44100,channels=2                        "
-	      "        ! audioconvert                                                 "
-	      "        ! vorbisenc name=a-encoder                                     "
-	      "        ! mux.                                                         "
-	      "                                                                       "
-	      "                                                                       "
-	      "          demux.%s                                                     "
-	      "        ! queue name=q-v-in                                            "
-	      "        ! mpeg2dec                                                     "
-	      "        ! videorate                                                    "
-	      "        ! video/x-raw-yuv,framerate=(fraction)%u/1                     "
-	      "        ! videobox left=%u right=%u top=%u bottom=%u                   "
-	      "        ! thdeinterlace                                                "
-	      "        ! videoscale method=bilinear                                   "
-	      "        ! video/x-raw-yuv,width=%u,height=%u                           "
-	      "        ! theoraenc name=v-encoder                                     "
-	      "        ! mux.                                                         ",
-	      j->priv->audio_pad_name,
-	      j->priv->audio_decoder,
+	pipe_desc = g_string_new ("");
+
+	/* muxer and filesink */
+	g_string_append (pipe_desc,
+	      "    oggmux name=mux                                           "
+	      "  ! identity name=outidentity                                 "
+	      "  ! filesink name=filesink                                    "
+	      "                                                              ");
+
+	/* source and demuxer */
+	g_string_append (pipe_desc,
+	      "    dvdreadsrc name=src                                       "
+	      "  ! dvddemux name=demux                                       "
+	      "                                                              ");
+
+	/* audio transcoder(s) */
+	{
+	  guint i;
+
+	  for (i = 0; i < j->audio_streams->len; ++i) {
+	    ThJobAudioStream *as;
+
+	    as = &g_array_index (j->audio_streams, ThJobAudioStream, i);
+
+	    if (j->priv->audio_all || j->priv->audio_stream_id == as->aid) {
+	      g_string_append_printf (pipe_desc,
+	          "          demux.%s                                        "
+	          "    ! queue name=q-a-in-%u                                "
+	          "    ! %s                                                  "
+	          "    ! audioconvert                                        "
+	          "    ! audioresample                                       "
+	          "    ! audio/x-raw-float,rate=44100,channels=2             "
+	          "    ! vorbisenc name=a-encoder-%u                         "
+	          "    ! mux.                                                "
+	          "                                                          ",
+	          as->pad_name, as->aid, j->priv->audio_decoder, as->aid);
+
+	      th_log ("Adding audio stream %u, pad = %s\n", i, as->pad_name);
+	    }
+	  }
+	}
+
+	/* video transcoder  */	
+	g_string_append_printf (pipe_desc,
+	      "      demux.%s                                                "
+	      "    ! queue name=q-v-in                                       "
+	      "    ! mpeg2dec                                                "
+	      "    ! thparsetter pixel-aspect-ratio=%u/%u                    "
+	      "    ! %s                                                      "
+	      "    ! videorate                                               "
+	      "    ! video/x-raw-yuv,framerate=(fraction)%u/%u               "
+	      "    ! videobox left=%u right=%u top=%u bottom=%u              "
+	      "    ! videoscale method=%s                              "
+	      "    ! queue max-size-buffers=5                                "
+	      "    ! video/x-raw-yuv,width=%u,height=%u                      "
+	      "    ! theoraenc name=v-encoder                                "
+	      "    ! mux.                                                    "
+	      "                                                              ",
 	      j->priv->video_pad_name,
-	      (guint) 25, /* fixed to 25 fps for now, sorry */
+	      j->video_info.aspect_num, j->video_info.aspect_denom,
+	      deinterlacer,
+	      j->video_info.fps_num, j->video_info.fps_denom,
 	      j->priv->crop_left, j->priv->crop_right, 
 	      j->priv->crop_top, j->priv->crop_bottom,
+	      scale_method,
 	      width, height);
 	
-	pipe = gst_parse_launch (pipe_desc, err);
+	pipe = gst_parse_launch (pipe_desc->str, err);
+	g_string_free (pipe_desc, TRUE);
+	pipe_desc = NULL;
 	
 	if ((err && *err) || pipe == NULL)
 	{
 		if (pipe)
 			gst_object_unref (GST_OBJECT (pipe));
-		g_free (pipe_desc);
 		return FALSE;
 	}
 	
 	j->priv->pipeline = pipe;
 
-	job_create_pipeline_setup_queues (GST_BIN (pipe));
+	job_create_pipeline_setup_queues (j);
 
 	{
 		/* to avoid 'dereferencing type-punned pointer' compiler warnings */
@@ -1928,11 +2027,7 @@ job_create_pipeline (ThJob *j, GError **err)
 
 	job_create_pipeline_setup_video_encoder (j);
 
-	/* vorbisenc */
-	th_bin_set_child_properties (GST_BIN (pipe), "a-encoder",
-	              "bitrate", j->priv->audio_bitrate * 1000,
-	              "managed", TRUE, 
-	              NULL);
+	job_create_pipeline_setup_audio_encoders (j);
 
 	/* oggmux */
 	th_bin_set_child_properties (GST_BIN (pipe), "mux",
@@ -1966,8 +2061,6 @@ job_create_pipeline (ThJob *j, GError **err)
 	gst_object_unref (out_identity);
 
 	gst_element_set_state (j->priv->pipeline, GST_STATE_PLAYING);
-
-	g_free (pipe_desc);
 	
 	return TRUE;
 }
@@ -2026,8 +2119,6 @@ th_job_run (ThJob *j, GError **err)
 		j->priv->pipeline = NULL;
 	}
 
-	g_free (j->priv->audio_pad_name);
-	g_free (j->priv->video_pad_name);
 	j->priv->audio_pad_name = NULL;
 	j->priv->video_pad_name = NULL;
 
@@ -2173,7 +2264,7 @@ th_job_set_filename_from_disc_title (ThJob *j, const gchar *disc_title)
 		disc_title_lc = g_utf8_strdown (disc_title, -1);
 		g_strdelimit (disc_title_lc, "\\/ ", '-');
 		
-		fn = g_strdup_printf ("%s/%s-%02u.ogg", homedir, disc_title_lc, j->priv->title_num + 1);
+		fn = g_strdup_printf ("%s/%s-%02u.ogv", homedir, disc_title_lc, j->priv->title_num + 1);
 		
 		g_free (disc_title_lc);
 	}
@@ -2185,7 +2276,7 @@ th_job_set_filename_from_disc_title (ThJob *j, const gchar *disc_title)
 		if (id && strchr (id, '-'))
 			*strchr (id, '-') = 0x00;
 
-		fn = g_strdup_printf ("%s/dvd-%s-%02u.ogg", homedir, (id) ? id : "", j->priv->title_num + 1);
+		fn = g_strdup_printf ("%s/dvd-%s-%02u.ogv", homedir, (id) ? id : "", j->priv->title_num + 1);
 		
 		g_free (id);
 	}
@@ -2203,7 +2294,7 @@ th_job_set_filename_from_disc_title (ThJob *j, const gchar *disc_title)
 
 void
 th_job_add_audio_stream (ThJob *j, guint aid, const gchar *pad_name,
-    const gchar *lang, const gchar *desc)
+    const gchar *lang, const gchar *desc, const gchar *decoder)
 {
 	ThJobAudioStream *as;
 	const gchar      *cur_lang;
@@ -2213,15 +2304,17 @@ th_job_add_audio_stream (ThJob *j, guint aid, const gchar *pad_name,
 	g_return_if_fail (lang != NULL);
 	g_return_if_fail (desc != NULL);
 	g_return_if_fail (pad_name != NULL);
+	g_return_if_fail (decoder != NULL);
 	
 	as = g_new0 (ThJobAudioStream, 1);
 	
 	as->aid      = aid;
-	as->lang     = g_strdup (lang);
-	as->desc     = g_strdup (desc);
-	as->pad_name = g_strdup (pad_name);
+	as->lang     = g_intern_string (lang);
+	as->desc     = g_intern_string (desc);
+	as->pad_name = g_intern_string (pad_name);
+	as->decoder  = g_intern_string (decoder);
 
-	j->priv->audio_streams = g_list_append (j->priv->audio_streams, as);
+	g_array_append_vals (j->audio_streams, as, 1);
 	
 	/* Get current locale; if none set, assume English */
 	cur_lang = g_getenv ("LANG");
@@ -2240,8 +2333,6 @@ th_job_add_audio_stream (ThJob *j, guint aid, const gchar *pad_name,
 		th_log ("[%u] Default Audio Stream: %s (aid = %u, pad = %s)", 
 		        j->priv->title_num + 1, as->lang, aid, as->pad_name);
 	}
-	
-	g_object_notify (G_OBJECT (j), "num-audio-streams");
 }
 
 /***************************************************************************
@@ -2256,11 +2347,12 @@ th_job_get_audio_stream (ThJob *j, guint n, guint *p_aid, const gchar **p_lang,
 	ThJobAudioStream *as;
 
 	g_return_val_if_fail (TH_IS_JOB (j), FALSE);
-	
-	as = (ThJobAudioStream*) g_list_nth_data (j->priv->audio_streams, n);
-	if (as == NULL)
+
+	if (n >= j->audio_streams->len)
 		return FALSE;
-	
+
+	as = &g_array_index (j->audio_streams, ThJobAudioStream, n);
+
 	if (p_aid)
 		*p_aid = as->aid;
 
@@ -2273,6 +2365,62 @@ th_job_get_audio_stream (ThJob *j, guint n, guint *p_aid, const gchar **p_lang,
 	return TRUE;
 }
 
+
+/***************************************************************************
+ *
+ *   th_job_add_subpicture_stream
+ *
+ ***************************************************************************/
+
+void
+th_job_add_subpicture_stream (ThJob * j, guint sid, const gchar * pad_name,
+    const gchar * lang)
+{
+	ThJobSubStream *ss;
+
+	g_return_if_fail (TH_IS_JOB (j));
+	g_return_if_fail (sid <= 99);
+	g_return_if_fail (lang != NULL);
+	g_return_if_fail (pad_name != NULL);
+	
+	ss = g_new0 (ThJobSubStream, 1);
+	
+	ss->sid      = sid;
+	ss->lang     = g_strdup (lang);
+	ss->pad_name = g_strdup (pad_name);
+
+	j->priv->sub_streams = g_list_append (j->priv->sub_streams, ss);
+	
+	g_object_notify (G_OBJECT (j), "num-sub-streams");
+}
+
+/***************************************************************************
+ *
+ *   th_job_get_subpicture_stream
+ *
+ ***************************************************************************/
+
+gboolean
+th_job_get_subpicture_stream (ThJob * j, guint n, guint * p_sid,
+    const gchar ** p_lang)
+{
+	ThJobSubStream *ss;
+
+	g_return_val_if_fail (TH_IS_JOB (j), FALSE);
+	
+	ss = (ThJobSubStream*) g_list_nth_data (j->priv->sub_streams, n);
+	if (ss == NULL)
+		return FALSE;
+	
+	if (p_sid)
+		*p_sid = ss->sid;
+
+	if (p_lang)
+		*p_lang = ss->lang;
+
+	return TRUE;
+}
+
 /***************************************************************************
  *
  *   th_job_get_file_size_range
@@ -2285,11 +2433,11 @@ th_job_get_file_size_range (ThJob *job, gdouble *p_min_size, gdouble *p_max_size
 	g_return_if_fail (TH_IS_JOB (job));
 
 	if (p_min_size)
-		*p_min_size = job_get_output_size_estimate (job, job->priv->audio_bitrate, 1, job->priv->title_length);
+		*p_min_size = job_get_output_size_estimate (job, 1);
 	
 	/* theoraenc currently only accepts video bitrates up to 2000 kbps (0.8.7) */
 	if (p_max_size)
-		*p_max_size = job_get_output_size_estimate (job, job->priv->audio_bitrate, 2000, job->priv->title_length);
+		*p_max_size = job_get_output_size_estimate (job, 2000);
 }
 
 
