@@ -73,7 +73,9 @@
 
 #define TH_UTILS_ERROR  g_quark_from_static_string("thoggen-utils-error")
 
+static gpointer        gui_thread;          /* NULL */
 static gpointer        log_text_buffer;     /* NULL */
+static GAsyncQueue    *pending_log_lines;   /* NULL */
 static GHashTable     *iso_639_ht;          /* NULL */
 
 /***************************************************************************
@@ -1200,6 +1202,8 @@ create_log_text_buffer (void)
 	GtkTextTagTable *ttt;
 	GtkTextTag      *tag;
 
+	gui_thread = g_thread_self ();
+
 	log_text_buffer = gtk_text_buffer_new (NULL);
 	ttt = gtk_text_buffer_get_tag_table (log_text_buffer);
 	tag = gtk_text_tag_new ("log-tag");
@@ -1215,6 +1219,21 @@ create_log_text_buffer (void)
 	g_atexit (cleanup_log_text_buffer);
 
 	g_object_add_weak_pointer (G_OBJECT (log_text_buffer), &log_text_buffer);
+
+	pending_log_lines = g_async_queue_new ();
+}
+
+static gboolean
+process_pending_log_lines (gpointer data)
+{
+  gchar *t;
+
+  if ((t = g_async_queue_try_pop (pending_log_lines))) {
+    th_log_add_line (t);
+    g_free (t);
+  }
+
+  return (g_async_queue_length (pending_log_lines) > 0);
 }
 
 /******************************************************************************
@@ -1231,6 +1250,17 @@ th_log_add_line (const gchar *txt)
 		GtkTextIter end;
 		gsize len;
 
+		/* if a GStreamer element throws a g_warning or something,
+		 * we might get called from one of the streaming threads,
+		 * ie. in a thread context we really shouldn't be doing any
+		 * Gtk/GUI things whatsoever; in this case, just queue the
+		 * message for later adding to the buffer */
+		if (log_text_buffer != NULL && gui_thread != g_thread_self ()) {
+		  g_async_queue_push (pending_log_lines, g_strdup (txt));
+		  g_idle_add (process_pending_log_lines, NULL);
+		  return;
+		}
+
 		if (log_text_buffer == NULL)
 			create_log_text_buffer ();
 
@@ -1524,7 +1554,10 @@ th_bin_set_child_properties (GstBin *bin, const gchar *child_name,
 	g_return_if_fail (prop1_name != NULL);
 
 	child = gst_bin_get_by_name (bin, child_name);
-	g_return_if_fail (child != NULL);
+	if (child == NULL) {
+		g_warning ("Pipeline contains no child named '%s'", child_name);
+		g_return_if_fail (child != NULL);
+	}
 
 	va_start (args, prop1_name);
 	g_object_set_valist (G_OBJECT (child), prop1_name, args);
@@ -1532,3 +1565,26 @@ th_bin_set_child_properties (GstBin *bin, const gchar *child_name,
 
 	gst_object_unref (child);
 }
+
+/***************************************************************************
+ *
+ *   th_simplify_fraction
+ *
+ ***************************************************************************/
+
+void
+th_simplify_fraction (guint * num, guint * denom)
+{
+  guint a, b, tmp;
+
+  a = *num;
+  b = *denom;
+  while (a) {    /* find greatest common divisor */
+    tmp = a;
+    a = b % tmp;
+    b = tmp;
+  }
+  *num = *num / b;
+  *denom = *denom / b;
+}
+
